searchData={"items":[{"type":"behaviour","title":"Permit","doc":"Permit is an extensible, DSL-less library allowing the coder to define authorization rules in plain Elixir.\n\nIt can run on its own, but is also integrated with widely used Elixir libraries and frameworks.","ref":"Permit.html"},{"type":"behaviour","title":"Libraries and repositories - Permit","doc":"The Permit library can run on its own as a standalone library, but it can also be used alongside Ecto and Phoenix integrations.\n\n  * [`Permit`](https://github.com/curiosum-dev/permit) - provides a syntax to define permissions to perform actions (defined as atoms) on objects (structs) by a specific user (subject) using functions or keyword lists, matching against an object's attributes.\n\n  * [`Permit.Ecto`](https://github.com/curiosum-dev/permit_ecto) - provides a resolver using Ecto to build and execute singular or collection Ecto queries based on defined permissions, also extending the syntax with a possibility to define more sophisticated permissions convertible to Ecto queries.\n\n  * [`Permit.Phoenix`](https://github.com/curiosum-dev/permit_phoenix) - uses `Permit` and `Permit.Ecto` to retrieve records via loader functions or queries generated by `Permit.Ecto` (if installed), based on data accessible in current context defined by a Plug `conn` or a LiveView `socket`.","ref":"Permit.html#module-libraries-and-repositories"},{"type":"behaviour","title":"Paradigm and Extensibility - Permit","doc":"At the core of authorization resolution, there's always the question of:\n* **What action** is being performed (for Phoenix, it's most likely a controller action)\n* **What subject** performs the action (usually, the current user)\n* **What object** the action is performed on\n\nOnce answers to these three questions are found, authorization or lack thereof is determined based on the set of permission definitions, defined as [expressions in disjunctive normal form (DNF)](https://en.wikipedia.org/wiki/Disjunctive_normal_form) expressions - that is, a set of sufficient conditions, with each condition defined as a conjunction of predicates, for example:\n```text\n   Subject     |   Action   | Object\n--------------------------------------------\nA **user** can | **update** | an **article**\n\n...if user's ID = article author's ID AND the article is not published,\n...if user's ID = article author's ID AND the article type is a live ticker,\n...if user's role is editor-in-chief AND the article is not published,\n...if user's role is editor-in-chief ID AND the article type is a live ticker,\n...or if the use has a super-admin role.\n```\n\nWhich, in Permit syntax, is translated to the following. Note the usage of pattern matching on the current user's (subject's) attributes, which allows to create function clauses for each user role. Permit does not enforce a specific structure of the `can/1` function, but as pattern matching usage is convenient in this case, it is naturally encouraged.\n```elixir\ndef can(%User{role: :editor_in_chief} = _current_user) do\n  permit()\n  |> update(Article, state: {:not, :published})\n  |> update(Article, type: :live_ticker)\nend\n\ndef can(%User{id: user_id} = _current_user) do\n  permit()\n  |> update(Article, author_id: user_id, state: {:not, :published})\n  |> update(Article, author_id: user_id, type: :live_ticker)\nend\n\ndef can(%User{id: user_id, role: :super_admin} = _current_user) do\n  permit()\n  |> update(Article)\nend\n```\n\nThe library is written with extensibility in mind. Analogously to Phoenix interoperatbility, the developer may define their own integration with different frameworks.\n\nFor more details on interoperability, see `Permit.ResolverBase`.","ref":"Permit.html#module-paradigm-and-extensibility"},{"type":"behaviour","title":"Configuration and usage - Permit","doc":"For more details on Ecto and Phoenix usage, visit [`permit_ecto`](https://hexdocs.pm/permit_ecto) and [`permit_phoenix`](https://hexdocs.pm/permit_phoenix) documentations, respectively.","ref":"Permit.html#module-configuration-and-usage"},{"type":"behaviour","title":"Configure & define your permissions - Permit","doc":"```elixir\ndefmodule MyApp.Authorization do\n  use Permit, permissions_module: MyApp.Permissions\nend\n\ndefmodule MyApp.Permissions do\n  use Permit.Permissions, actions_module: Permit.Actions.CrudActions\n\n  def can(%{role: :admin} = user) do\n    permit()\n    |> all(MyApp.Blog.Article)\n  end\n\n  def can(%{id: user_id} = user) do\n    permit()\n    |> all(MyApp.Blog.Article, author_id: user_id)\n    |> read(MyApp.Blog.Article)\n  end\n\n  def can(user), do: permit()\nend\n```\nNote that in the permission definitions module the `read` function is generated based on configuration provided as the `:actions_module` option - in this case, `CrudActions` generates `create`, `read`, `update` and `delete`. For more on this, see `Permit.Actions` and `Permit.Permissions`.","ref":"Permit.html#module-configure-define-your-permissions"},{"type":"behaviour","title":"Check a user's authorization to perform an action on a resource - Permit","doc":"```elixir\niex(1)> import MyApp.Authorization\niex(2)> can(%MyApp.User{id: 1}) |> read?(%MyApp.Article{author_id: 1})\ntrue\niex(3)> can(%MyApp.User{id: 1}) |> read?(%MyApp.Article{author_id: 2})\ntrue\niex(4)> can(%MyApp.User{id: 1}) |> update?(%MyApp.Article{author_id: 2})\nfalse\niex(4)> can(%MyApp.User{role: :admin}) |> delete?(%MyApp.Article{author_id: 2})\ntrue\n```\nFunctions such as `MyApp.Authorization.read?/2`, `MyApp.Authorization.update?/2`, etc. are also generated based on the `:actions_module` option. See more in `Permit.Actions`.","ref":"Permit.html#module-check-a-user-s-authorization-to-perform-an-action-on-a-resource"},{"type":"callback","title":"Permit.resolver_module/0","doc":"","ref":"Permit.html#c:resolver_module/0"},{"type":"function","title":"Permit.verify_record/3","doc":"Verify if the authorization has the given action on the given resource.\n\nMost of the time this function will not be called directly, but rather through the `do?/3` delegation\nvia a module that uses `Permit`.","ref":"Permit.html#verify_record/3"},{"type":"function","title":"Examples - Permit.verify_record/3","doc":"```elixir\niex(1)> import MyApp.Authorization\niex(2)> can(%MyApp.User{id: 1}) |> do?(:read, %MyApp.Article{author_id: 1})\ntrue\n```","ref":"Permit.html#verify_record/3-examples"},{"type":"protocol","title":"Permit.SubjectMapping","doc":"Allows mapping subject structure into one or more distinct subject records to denote that _any_ of these must be authorized to perform an action.\n\nDefault mapping (implementation for `Any`) returns `[subject]` for a given `subject`, so all permissions will be checked against a single subject structure.\n\nExample - scenario of a multi-user session, in which any signed-in user must have permissions for authorization to be granted:\n```elixir\ndefmodule User, do: defstruct id: 1, name: \"foo\", role:\ndefmodule UserSession,\n  do: defstruct session_id: 1, current_users: []\n\ndefimpl Permit.SubjectMapping, for: UserSession do\n  def subjects(%UserSession{current_users: current_users}) do\n    current_users\n  end\nend\n\ndefmodule Permissions do\n  use Permit.Ecto.Permissions, actions_module: Permit.Actions.CrudActions\n\n  def can(%User{role: :reader} = user), do: permit()\n  def can(%User{role: :auditor} = user), do: permit()\nend\n\ndefmodule Authorization, do: use Permit, permissions_module: Permissions\n```\n\niex(1)> Authorization.can(%UserSession{users: [%User{role: :reader}, %User{role: :auditor}]})\niex(2)> |> Authorization.update?(%Article{})\ntrue","ref":"Permit.SubjectMapping.html"},{"type":"function","title":"Permit.SubjectMapping.subjects/1","doc":"","ref":"Permit.SubjectMapping.html#subjects/1"},{"type":"type","title":"Permit.SubjectMapping.t/0","doc":"All the types that implement this protocol.","ref":"Permit.SubjectMapping.html#t:t/0"},{"type":"behaviour","title":"Permit.Actions","doc":"","ref":"Permit.Actions.html"},{"type":"behaviour","title":"Overview - Permit.Actions","doc":"The `Permit.Actions` behaviour defines an exhaustive set of actions that can be performed on resources in the business domain.\n\nActions can be grouped using the `c:grouping_schema/0` callback, which is useful when the developer intends to define permissions that imply that multiple actions will be authorized. For instance, in the `Permit.Phoenix` library, granting the `:read` permission results in the `:index` and `:show` actions being authorized.\n\nPermit includes a predefined `Permit.Actions.CrudActions` module that defines four basic CRUD actions: `create`, `read`, `update` and `delete`.\n\n`Permit.Permissions` generates functions for each of defined CRUD action names, as shorthands to define permissions to perform them - as well as an `all` function to permit all possible actions.\n\nMoreover, the `c:singular_actions/0` callback can be implemented to declare which actions are of singular nature, and which are of plural nature - for instance, an `:index` action is typically plural and a `:show` action is typically singular. This means that if automatic preloading mechanisms are in place, `:index` will load many records, and `:show` will load a single record.","ref":"Permit.Actions.html#module-overview"},{"type":"behaviour","title":"Example - Permit.Actions","doc":"For instance, to make an `:index` action separate from a `:read` action, you may define an additional `:open` action and define `:index` as requiring only `:read`, and `:show` as requiring both `:open` and `:read`.\n\n    defmodule MyApp.Actions do\n      use Permit.Actions\n\n      @impl Permit.Actions\n      def grouping_schema do\n        crud_grouping() # Includes :create, :read, :update and :delete\n        |> Map.merge(%{\n          index: [:read],\n\n          # This is a 'plain' action not dependent on any other one, i.e. permission to these can be assigned directly\n          open: []\n\n          # This indicates that for the :show action to be performed, the :read AND open permissions must be granted.\n          show: [:read, :open]\n        })\n      end\n    end\n\nDeclaring permissions basing on defined actions:\n\n    defmodule MyApp.Permissions do\n      use Permit.Permissions, actions_module: MyApp.Actions\n\n      # An admin will be able to perform all actions on Note\n      def can(%User{role: :admin} = _user) do\n        permit()\n        |> all(Note)\n      end\n\n      # A user will be able\n      def can(%User{id: user_id} = _user) do\n        permit()\n        |> read(Note)\n        |> open(Note, user_id: user_id)\n      end\n\n      def can(_), do: permit()\n    end","ref":"Permit.Actions.html#module-example"},{"type":"callback","title":"Permit.Actions.grouping_schema/0","doc":"Used for mapping business domain _actions_ to _conjunctions of permissions_ required to perform them.\n\nIn the example below, granting `:read` and `:open` means that `:show` action can be performed. Note, though, that this is a one-way implication - `:show` permission can be granted, but it does not imply that `:read` or `:open` are granted.","ref":"Permit.Actions.html#c:grouping_schema/0"},{"type":"callback","title":"Example - Permit.Actions.grouping_schema/0","doc":"@impl Permit.Actions\n    def grouping_schema do\n      crud_grouping() # Includes :create, :read, :update and :delete\n      |> Map.merge(%{\n        index: [:read],\n\n        # This is a 'plain' action not dependent on any other one, i.e. permission to these can be assigned directly\n        open: []\n\n        # This indicates that for the :show action to be performed, the :read AND :open permissions must be granted.\n        show: [:read, :open]\n      })\n    end","ref":"Permit.Actions.html#c:grouping_schema/0-example"},{"type":"callback","title":"Permit.Actions.singular_actions/0","doc":"Declares which actions are singular, and which are plural.\n\nFor instance, an `:index` action is typically plural and a `:show` action is typically singular. This means that if automatic preloading mechanisms are in place, `:index` will load many records, and `:show` will load a single record.","ref":"Permit.Actions.html#c:singular_actions/0"},{"type":"callback","title":"Example - Permit.Actions.singular_actions/0","doc":"@impl Permit.Actions\n    def singular_actions do\n      crud_singular() ++ [:show, :open]\n    end","ref":"Permit.Actions.html#c:singular_actions/0-example"},{"type":"module","title":"Permit.Actions.CrudActions","doc":"Extends the predefined `Permit.Actions` module and defines the following action mapping:\n\n| **Action** | **Required permission** |\n|------------|-------------------------|\n| `:create`  | itself                  |\n| `:read  `  | itself                  |\n| `:delete`  | itself                  |\n| `:update`  | itself                  |\n\nFor more information on defining and mapping actions, see `Permit.Actions` documentation.","ref":"Permit.Actions.CrudActions.html"},{"type":"function","title":"Permit.Actions.CrudActions.crud_grouping/0","doc":"Convenience function defining the basic CRUD (create, read, update, delete) actions.","ref":"Permit.Actions.CrudActions.html#crud_grouping/0"},{"type":"function","title":"Permit.Actions.CrudActions.crud_singular/0","doc":"Convenience function returning actions that are singular in the most basic CRUD setup, in which case\nall of: `:create`, `:read`, `:update` and `:delete` are singular.","ref":"Permit.Actions.CrudActions.html#crud_singular/0"},{"type":"module","title":"Permit.Actions.Forest","doc":"Encapsulates the directed acyclic graph built from permissions defined using `Permit.Permissions` and provides traversal functions.\n\nPart of the private API, subject to changes and not to be used on the application level.","ref":"Permit.Actions.Forest.html"},{"type":"type","title":"Permit.Actions.Forest.t/0","doc":"","ref":"Permit.Actions.Forest.html#t:t/0"},{"type":"type","title":"Permit.Actions.Forest.vertex/0","doc":"","ref":"Permit.Actions.Forest.html#t:vertex/0"},{"type":"module","title":"Permit.Actions.Traversal","doc":"Intended for usage by `Permit.Resolver` and derivatives. Traverses the tree of action definitions\ndefined with `Permit.Actions` to construct a result based on a given context. The context includes:\n* the action name to be authorized\n* a function that returns a value indicating whether an authorization condition is satisfied\n* a function for the conjunction of values (\"AND\")\n* a function for the disjunction values (\"OR\")\n\nDepending on the intended resolution product, these functions are different.\n\nIn `Permit`'s vanilla resolver, each record must be checked against authorization conditions\nfor its inclusion in the results. Thus, the passed value function returns a boolean (based on\nwhether there is permission to given action), and the conjunctor and disjunctor are `Enum.all?/1`\nand `Enum.any?/1`, respectively.\n\nIn `Permit.Ecto` resolver, which constructs query structures based on authorization conditions,\nthe values are Ecto dynamic query expressions converted from the authorization conditions,\nwhereas the conjunctor joins values using SQL `AND`, and the disjunctor uses SQL `OR`.\n\nPart of the private API, subject to changes and not to be used on the application level.","ref":"Permit.Actions.Traversal.html"},{"type":"behaviour","title":"Permit.Permissions","doc":"Defines the application's permission set. When used with `Permit.Ecto`, one should use `Permit.Ecto.Permissions` instead of `Permit.Permissions`.\n\nThe behaviour defines the `c:can/1` callback, which must be implemented for defining permissions for a given subject.\n\nThe module's `__using__/1` macro creates functions for each action defined in the module specified as the macro's option, defaulting to `Permit.Actions.CrudActions`.","ref":"Permit.Permissions.html"},{"type":"behaviour","title":"Usage - Permit.Permissions","doc":"A very simple usage example:\n```\ndefmodule MyApp.Permissions do\n  use Permit.Permissions, actions_module: Permit.Actions.CrudActions\n\n  @impl true\n  def can(%MyApp.User{role: :admin}) do\n    permit()\n    |> all(Article)\n  end\n\n  def can(%MyApp.User{id: user_id}) do\n    permit()\n    |> read(Article)\n    |> all(Article, author_id: user_id)\n  end\n\n  def can(_), do: permit()\nend\n```","ref":"Permit.Permissions.html#module-usage"},{"type":"behaviour","title":"Named action functions - Permit.Permissions","doc":"Each action defined in the `:actions_module` results in a 2-, 3-, and 4-arity function being generated.\n\nFor instance, if a `:read` action is defined, there are the following calls available to grant the `:read` permission on a given resource type:\n* `read/2` function - grants permission without additional conditions\n* `read/3` function - with conditions defined using keywords and operators (see below),\n* `read/4` macro - with conditions defined using keywords, operators and bindings (see below).","ref":"Permit.Permissions.html#module-named-action-functions"},{"type":"behaviour","title":"Example - Permit.Permissions","doc":"def can(%User{id: user_id}) do\n      permit()\n      |> read(Article, author_id: user_id)\n      |> vote(Article, vote_count: {:<=, 100})\n      |> review(Article, [user, article], user.level >= article.level)\n    end\n\n## `permission_to` functions\n\nInstead of action names, if more convenient, `permission_to` can be used, and the action name passed as an argument.","ref":"Permit.Permissions.html#module-example"},{"type":"behaviour","title":"Example - Permit.Permissions","doc":"def can(%User{id: user_id}) do\n      permit()\n      |> permission_to(:read, Article, author_id: user_id)\n      |> permission_to(:vote, Article, vote_count: {:<=, 100})\n      |> permission_to(:review, Article, [user, article], user.level >= article.level)\n    end\n\n## `all` functions\n\nIn order to grant the user permission to all defined actions, use the `all` functions.","ref":"Permit.Permissions.html#module-example-1"},{"type":"behaviour","title":"Example - Permit.Permissions","doc":"def can(%User{id: user_id}) do\n      permit()\n      |> all(Article, author_id: user_id)\n      |> all(Article, vote_count: {:<=, 100})\n      |> all(Article, [user, article], user.level >= article.level)\n    end","ref":"Permit.Permissions.html#module-example-2"},{"type":"callback","title":"Permit.Permissions.can/1","doc":"","ref":"Permit.Permissions.html#c:can/1"},{"type":"type","title":"Permit.Permissions.conditions_by_action_and_resource/0","doc":"","ref":"Permit.Permissions.html#t:conditions_by_action_and_resource/0"},{"type":"function","title":"Permit.Permissions.parse_conditions/3","doc":"","ref":"Permit.Permissions.html#parse_conditions/3"},{"type":"type","title":"Permit.Permissions.t/0","doc":"","ref":"Permit.Permissions.html#t:t/0"},{"type":"module","title":"Permit.Permissions.DisjunctiveNormalForm","doc":"Describes conditions written as logical formula in disjunctive normal form.\n\nExample of a compound condition in DNF is:\n```\n(condition 1 AND condition 2)\nOR (condition 3)\nOR (condition 4 AND condition 5 AND condition 6)\n```\n\nPart of the private API, subject to changes and not to be used on the\napplication level.","ref":"Permit.Permissions.DisjunctiveNormalForm.html"},{"type":"function","title":"Permit.Permissions.DisjunctiveNormalForm.add_clauses/2","doc":"","ref":"Permit.Permissions.DisjunctiveNormalForm.html#add_clauses/2"},{"type":"function","title":"Permit.Permissions.DisjunctiveNormalForm.any_satisfied?/3","doc":"","ref":"Permit.Permissions.DisjunctiveNormalForm.html#any_satisfied?/3"},{"type":"function","title":"Permit.Permissions.DisjunctiveNormalForm.concatenate/2","doc":"","ref":"Permit.Permissions.DisjunctiveNormalForm.html#concatenate/2"},{"type":"function","title":"Permit.Permissions.DisjunctiveNormalForm.new/1","doc":"","ref":"Permit.Permissions.DisjunctiveNormalForm.html#new/1"},{"type":"type","title":"Permit.Permissions.DisjunctiveNormalForm.t/0","doc":"","ref":"Permit.Permissions.DisjunctiveNormalForm.html#t:t/0"},{"type":"module","title":"Permit.Permissions.ParsedCondition","doc":"Represents the product of parsing a condition by a function implementing\nthe `c:Permit.Permissions.can/1` callback.\n\nA condition parsed by Permit's rule syntax parser contains:\n* condition semantics, that is: a function that allows for checking\n  whether the condition is satisfied\n* an indication of whether it is negated (i.e. a condition defined as\n  `{:not, ...}`)\n* metadata (`:private`), which can be used by alternative parsers (e.g.\n  `Permit.Ecto.Permissions` puts dynamic query constructors there)\n\nPart of the private API, subject to changes and not to be used on the\napplication level.","ref":"Permit.Permissions.ParsedCondition.html"},{"type":"type","title":"Permit.Permissions.ParsedCondition.condition_type/0","doc":"","ref":"Permit.Permissions.ParsedCondition.html#t:condition_type/0"},{"type":"type","title":"Permit.Permissions.ParsedCondition.t/0","doc":"","ref":"Permit.Permissions.ParsedCondition.html#t:t/0"},{"type":"module","title":"Permit.Permissions.ParsedConditionList","doc":"Encapsulates the list of conditions having been parsed by an implementation of\n`Permit.Permissions`.\n\nThis list is to be treated as a **conjunction** of conditions. The logical model\nof Permit implies that these conjunctions are then linked in an `OR` manner\nto form a disjunctive normal form - see more in `Permit.Permissions.DisjunctiveNormalForm`.\n\nPart of the private API, subject to changes and not to be used on the\napplication level.","ref":"Permit.Permissions.ParsedConditionList.html"},{"type":"type","title":"Permit.Permissions.ParsedConditionList.t/0","doc":"","ref":"Permit.Permissions.ParsedConditionList.html#t:t/0"},{"type":"module","title":"Permit.Operators","doc":"Represents the list of operators for usage in defining permission conditions, for example\n`Permit.Operators.Eq` defines the equality operator usable as `:==` or alternatively\n`:eq`, with given semantics.\n\nPart of the private API, subject to changes and not to be used on the application level.","ref":"Permit.Operators.html"},{"type":"macro","title":"Permit.Operators.all/0","doc":"","ref":"Permit.Operators.html#all/0"},{"type":"macro","title":"Permit.Operators.eq/0","doc":"","ref":"Permit.Operators.html#eq/0"},{"type":"macro","title":"Permit.Operators.eq_operators/0","doc":"","ref":"Permit.Operators.html#eq_operators/0"},{"type":"function","title":"Permit.Operators.get/1","doc":"","ref":"Permit.Operators.html#get/1"},{"type":"macro","title":"Permit.Operators.with_options/0","doc":"","ref":"Permit.Operators.html#with_options/0"},{"type":"module","title":"Permit.Operators.Eq","doc":"Equality operator, accessible via `:==` and `:eq`.","ref":"Permit.Operators.Eq.html"},{"type":"module","title":"Permit.Operators.Ge","doc":"Greater-than-or-equal operator, accessible via `:>=` and `:ge`.","ref":"Permit.Operators.Ge.html"},{"type":"behaviour","title":"Permit.Operators.GenOperator","doc":"Generic operator behaviour. For each operator, it allows defining:\n* the main symbol (e.g. `:==`)\n* alternative and equivalent symbols (e.g. `:eq`)\n* semantics function builders, returning functions to determine whether the operator is truthy or falsy.\n\nPart of the private API, subject to changes and not to be used on the application level.","ref":"Permit.Operators.GenOperator.html"},{"type":"callback","title":"Permit.Operators.GenOperator.alternatives/0","doc":"","ref":"Permit.Operators.GenOperator.html#c:alternatives/0"},{"type":"type","title":"Permit.Operators.GenOperator.condition_fn/0","doc":"","ref":"Permit.Operators.GenOperator.html#t:condition_fn/0"},{"type":"type","title":"Permit.Operators.GenOperator.operator_result/0","doc":"","ref":"Permit.Operators.GenOperator.html#t:operator_result/0"},{"type":"callback","title":"Permit.Operators.GenOperator.semantics/1","doc":"","ref":"Permit.Operators.GenOperator.html#c:semantics/1"},{"type":"callback","title":"Permit.Operators.GenOperator.semantics/2","doc":"","ref":"Permit.Operators.GenOperator.html#c:semantics/2"},{"type":"callback","title":"Permit.Operators.GenOperator.symbol/0","doc":"","ref":"Permit.Operators.GenOperator.html#c:symbol/0"},{"type":"module","title":"Permit.Operators.Gt","doc":"Greater-than operator, accessible via `:>` and `:gt`.","ref":"Permit.Operators.Gt.html"},{"type":"module","title":"Permit.Operators.Ilike","doc":"`ILIKE` operator, accessible via `:ilike`. Semantically equivalent to matching a regular expression built\nfrom a SQL-like syntax (e.g. `\"FOO%BAR\"` pattern is equivalent to `~r/FOO.*BAR/`). Case insensitive.","ref":"Permit.Operators.Ilike.html"},{"type":"module","title":"Permit.Operators.In","doc":"List inclusion operator, accessible via `:in`. Implemented using `Kernel.in/2`.","ref":"Permit.Operators.In.html"},{"type":"module","title":"Permit.Operators.IsNil","doc":"Nil equality operator operator, accessible via `:is_nil` or `:nil?`.","ref":"Permit.Operators.IsNil.html"},{"type":"module","title":"Permit.Operators.Le","doc":"Less-than-or-equal operator, accessible via `:le`, `:<=`.","ref":"Permit.Operators.Le.html"},{"type":"module","title":"Permit.Operators.Like","doc":"`LIKE` operator, accessible via `:like`. Semantically equivalent to matching a regular expression built\nfrom a SQL-like syntax (e.g. `\"FOO%BAR\"` pattern is equivalent to `~r/FOO.*BAR/`). Case sensitive.","ref":"Permit.Operators.Like.html"},{"type":"module","title":"Permit.Operators.Lt","doc":"Less-than operator, accessible via `:<` and `:lt`.","ref":"Permit.Operators.Lt.html"},{"type":"module","title":"Permit.Operators.Match","doc":"Regular expression matching operator, accessible via `:=~` and `:match`.","ref":"Permit.Operators.Match.html"},{"type":"module","title":"Permit.Operators.Neq","doc":"Not-equal operator, accessible via `:!=` and `:neq`.","ref":"Permit.Operators.Neq.html"},{"type":"module","title":"Permit.Resolver","doc":"Basic implementation of `Permit.ResolverBase` behaviour. Resolves and checks authorization of records or lists of records based on provided loader functions and parameters.\n\nFor a resolver implementation using Ecto for fetching resources, see `Permit.Ecto.Resolver` from the `permit_ecto` library.\n\nThis module is to be considered a private API of the authorization framework.\nIt should not be directly used by application code, but rather by wrappers\nproviding integration with e.g. Plug or LiveView.","ref":"Permit.Resolver.html"},{"type":"function","title":"Permit.Resolver.authorize_and_preload_all!/5","doc":"","ref":"Permit.Resolver.html#authorize_and_preload_all!/5"},{"type":"function","title":"Permit.Resolver.authorize_and_preload_one!/5","doc":"","ref":"Permit.Resolver.html#authorize_and_preload_one!/5"},{"type":"function","title":"Permit.Resolver.authorized?/4","doc":"","ref":"Permit.Resolver.html#authorized?/4"},{"type":"behaviour","title":"Permit.ResolverBase","doc":"Provides a basis for building _resolver modules_. A resolver is conceptually a module containing functions answering the following questions:\n\n* Given current **permission configuration**, is a **subject** authorized to perform a certain **action** on a given **resource**?\n* Given current **permission configuration**, a certain **subject**, an **action** and a **resource**, as well as an **execution context** (e.g. including controller parameters, loader functions, etc.), load a resource (e.g. by ID) and check whether the subject is authorized to perform the action on the resource.\n* Given current **permission configuration**, a certain **subject**, an **action** and a **resource**, as well as an **execution context**, load all resources on which the subject can perform the action.\n\nThe `Permit.ResolverBase` module implements `authorized?/4`, `authorize_and_preload_one!/5` and `authorize_and_preload_all!/5` functions to check against the permissions and provide a uniform API for the outcome of the resolution.\n\nCreating a resolver (see `Permit.Ecto.Resolver` as an example) requires the developer to implement the `c:resolve/6` callback that fetches data to be authorized against.\n\nReplacing the standard resolver (`Permit.Resolver`) with a more specific one (e.g. `Permit.Ecto.Resolver` from the `permit_ecto` library) is done by the usage of a different resolver module (e.g. `Permit.Ecto.Resolver`) instead of `Permit.Resolver` in the `__using__/1` macro of the module which implements the `Permit` behaviour. The `resolver_module/0` function has to be overridden to point to the new resolver (see `Permit.Ecto` for sample usage).","ref":"Permit.ResolverBase.html"},{"type":"callback","title":"Permit.ResolverBase.resolve/6","doc":"Implement to define a resolver's behavior.\n\nThe callback takes arguments in the following order:\n* `subject` typically takes the current user (or a record that `Permit.SubjectMapping` maps it to),\n* `authorization_module` takes the application's authorization configuration (i.e. the module that calls `use Permit` or `use Permit.Ecto`, or any other module with `Permit` behaviour),\n* `resource_module` takes the resource module - often, in Ecto applications, it's an Ecto schema,\n* `action_group` takes a name of an action, e.g. in Phoenix it's taken from a controller `conn` or a LiveView `socket`'s `live_action`\n* `meta` - depending on the resolver's needs, it will carry metadata such as loader functions, Ecto query processing functions, controller params, etc. - this generally is private to the integration library a developer is creating,\n* `arity` - it takes `:all` if plural resolution is to be performed (e.g. `:index`), and `:one` if singular resolution is performed (e.g. `:show`)\n\nThe callback implementation should use the arguments, including authorization context and application-specific context (particularly in `:meta` key), to resolve and return a record or records according to the spec.\n\nReturned value:\n* `{:authorized, object}` in a singular action if a record is found and authorization is granted,\n* `{:authorized, [object]}` in a plural action if authorization to given action is granted - and it is assumed that the resolver filters out records that are not authorized,\n* `:not_found` in a singular action if no record found, and thus authorization cannot be checked _(note that in plural actions `{:authorized, []}` should be returned)\n* `:unauthorized` in a singular action if a record is found but authorization check is negative, or in a singular or plural action if the action itself is not authorized at all.","ref":"Permit.ResolverBase.html#c:resolve/6"},{"type":"module","title":"Permit.Types","doc":"Includes type definitions used across the codebase of `Permit`, as well as its extensions.","ref":"Permit.Types.html"},{"type":"type","title":"Permit.Types.action_group/0","doc":"An action for which authorization is verified.","ref":"Permit.Types.html#t:action_group/0"},{"type":"type","title":"Permit.Types.authorization_module/0","doc":"Represents the application's main authorization module - the one that calls `use Permit` or `use Permit.Ecto`.","ref":"Permit.Types.html#t:authorization_module/0"},{"type":"type","title":"Permit.Types.id/0","doc":"A resource identifier, practically always being an integer or a string (e.g. a UUID).","ref":"Permit.Types.html#t:id/0"},{"type":"type","title":"Permit.Types.loader/0","doc":"A function used for preloading records by the resolver based on a resolution context.","ref":"Permit.Types.html#t:loader/0"},{"type":"type","title":"Permit.Types.object/0","doc":"A struct instance for a business level entity. Its type is the authorization module.","ref":"Permit.Types.html#t:object/0"},{"type":"type","title":"Permit.Types.object_or_resource_module/0","doc":"An object or a resource module can be used when asking for a specific permission.","ref":"Permit.Types.html#t:object_or_resource_module/0"},{"type":"type","title":"Example - Permit.Types.object_or_resource_module/0","doc":"```\n    can?(%User{role: :admin})\n    |> read?(Article)\n\n    can?(%User{role: :admin})\n    |> read?(%Article{id: 5})\n    ```","ref":"Permit.Types.html#t:object_or_resource_module/0-example"},{"type":"type","title":"Permit.Types.permissions/0","doc":"Encapsulates the permissions configuration for the application's business domain.","ref":"Permit.Types.html#t:permissions/0"},{"type":"type","title":"Permit.Types.permissions_code/0","doc":"Will generate code delegating to functions that return `Permit.Permissions`.","ref":"Permit.Types.html#t:permissions_code/0"},{"type":"type","title":"Permit.Types.resolution_context/0","doc":"Represents extra data for the purpose of resolving and preloading records by resolvers (`t:resolver_module/0`). It can include loader functions, query builder functions, controller parameters, etc. - it depends on the specifics of a resolver and is not meant for public usage.","ref":"Permit.Types.html#t:resolution_context/0"},{"type":"type","title":"Permit.Types.resolver_module/0","doc":"Represents a resolver module - the one that implements the `Permit.ResolverBase` behaviour, typically via `use Permit.ResolverBase`.","ref":"Permit.Types.html#t:resolver_module/0"},{"type":"type","title":"Permit.Types.resource_module/0","doc":"Represents a resource module that the authorization will be checked for. Typically, it is a struct representing a model of a business level entity, e.g. `Article` (not `%Article{}`).","ref":"Permit.Types.html#t:resource_module/0"},{"type":"type","title":"Permit.Types.struct_field/0","doc":"A name of a struct's field - typically, in structs such as Ecto schemas, etc., it will be an atom.","ref":"Permit.Types.html#t:struct_field/0"},{"type":"type","title":"Permit.Types.subject/0","doc":"Typically represents a current user in a given context.","ref":"Permit.Types.html#t:subject/0"},{"type":"module","title":"Permit.Types.ConditionTypes","doc":"Contains types of conditions that can be defined in the application's permission definition module (`Permit.Permissions` or e.g. for Ecto `Permit.Ecto.Permissions`).\n\nExtensions like `Permit.Ecto` will typically provide their own replacements for these types.","ref":"Permit.Types.ConditionTypes.html"},{"type":"type","title":"Permit.Types.ConditionTypes.boolean_condition/0","doc":"","ref":"Permit.Types.ConditionTypes.html#t:boolean_condition/0"},{"type":"type","title":"Permit.Types.ConditionTypes.condition/0","doc":"","ref":"Permit.Types.ConditionTypes.html#t:condition/0"},{"type":"type","title":"Permit.Types.ConditionTypes.condition_or_conditions/0","doc":"","ref":"Permit.Types.ConditionTypes.html#t:condition_or_conditions/0"},{"type":"type","title":"Permit.Types.ConditionTypes.fn1_condition/0","doc":"","ref":"Permit.Types.ConditionTypes.html#t:fn1_condition/0"},{"type":"type","title":"Permit.Types.ConditionTypes.fn2_condition/0","doc":"","ref":"Permit.Types.ConditionTypes.html#t:fn2_condition/0"},{"type":"type","title":"Permit.Types.ConditionTypes.keyword_equality_condition/0","doc":"","ref":"Permit.Types.ConditionTypes.html#t:keyword_equality_condition/0"},{"type":"exception","title":"Permit.CycledDefinitionError","doc":"Raised when action groupings defined in a `Permit.Actions` implementation result in a circular dependency.","ref":"Permit.CycledDefinitionError.html"},{"type":"exception","title":"Permit.UndefinedActionError","doc":"Raised when a given action is not implemented in the actions module (implementing `Permit.Actions`).","ref":"Permit.UndefinedActionError.html"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.38.2"}}